@@ -111,7 +111,7 @@ static void
 versionInfo(FILE *fp)
 {
     fprintf(fp, "Upmpdcli %s %s\n",
-           UPMPDCLI_PACKAGE_VERSION, LibUPnP::versionString().c_str());
+            UPMPDCLI_PACKAGE_VERSION, LibUPnP::versionString().c_str());
 }
 
 static void
@@ -167,6 +167,10 @@ static void onsig(int)
 {
     LOGDEB("Got sig" << endl);
     for (auto& dev : devs) {
+        // delete has a tendancy to crash (it works most of the time
+        // though). Anyway, we're exiting, so just call shouldExit()
+        // which will send the byebyes.
+        //delete dev;
         dev->shouldExit();
     }
     if (mpdclip) {
@@ -383,7 +387,9 @@ int main(int argc, char *argv[])
         g_config->get("ohmodelname", ohProductDesc.model.name);
         g_config->get("ohmodelinfo", ohProductDesc.model.info);
         g_config->get("ohmodelurl", ohProductDesc.model.url);
+        // imageUri was mistake, keep compat and override with imageuri if set
         g_config->get("ohmodelimageUri", ohProductDesc.model.imageUri);
+        g_config->get("ohmodelimageuri", ohProductDesc.model.imageUri);
         g_config->get("ohproductname", ohProductDesc.product.name);
         g_config->get("ohproductinfo", ohProductDesc.product.info);
         g_config->get("ohproducturl", ohProductDesc.product.url);
@@ -391,18 +397,18 @@ int main(int argc, char *argv[])
         g_config->get("ohproductroom", ohProductDesc.room);
         // ProductName is set to ModelName by default
         if (ohProductDesc.product.name.empty()) {
-          ohProductDesc.product.name = ohProductDesc.model.name;
+            ohProductDesc.product.name = ohProductDesc.model.name;
         }
         // ProductRoom is set to "Main Room" by default
         if (ohProductDesc.room.empty()) {
-          ohProductDesc.room = "Main Room";
+            ohProductDesc.room = "Main Room";
         }
 
         g_config->get("scsenderpath", senderpath);
         if (g_config->get("scsendermpdport", value))
             sendermpdport = atoi(value.c_str());
 
-        g_lumincompat = g_config->getBool("lumincompat", false);
+        g_lumincompat = g_config->getBool("lumincompat", g_lumincompat);
     } else {
         // g_configfilename is empty. Create an empty config anyway
         g_config = new ConfSimple(string(), 1, true);
@@ -484,7 +490,7 @@ int main(int argc, char *argv[])
             LOGFAT("Can't write pidfile: " << pidfile.getreason() << endl);
             return 1;
         }
-    if (opts.cachedir.empty())
+        if (opts.cachedir.empty())
             opts.cachedir = "/var/cache/upmpdcli";
     } else if (runas == geteuid()) {
         // Already running as upmpdcli user. There are actually 2
@@ -498,10 +504,10 @@ int main(int argc, char *argv[])
         // package was installed. One way to fix this if this if it is
         // wrong is to set cachedir in the configuration file
         // (opts.cachedir will be non-empty then).
-    if (opts.cachedir.empty())
+        if (opts.cachedir.empty())
             opts.cachedir = "/var/cache/upmpdcli";
     } else {
-    if (opts.cachedir.empty())
+        if (opts.cachedir.empty())
             opts.cachedir = path_cat(path_tildexpand("~") , "/.cache/upmpdcli");
     }
 
@@ -778,7 +784,6 @@ int main(int argc, char *argv[])
     if (!msonly) {
         LOGDEB("Renderer event loop" << endl);
         mediarenderer->startnoloops();
-        mpdclip->startEventLoop();
     }
 
     pause();
