@@ -18,6 +18,7 @@
 import os
 import time
 import re
+from recoll import recoll
 
 from timeit import default_timer as timer
 from uprclutils import audiomtypes, docfolder, uplog
@@ -36,7 +37,7 @@ import uprclinit
 # The keys must stay same as what Minim uses as we filter them with
 # indexTags from minim config
 #
-# Note: artist is actually doc.albumartist if set, else doc.artist and
+# Note: artist is actually doc["albumartist"] if set, else doc["artist"] and
 # is processed separately
 #
 # Only a smaller set of entries is normally used, as filtered by indexTags
@@ -49,7 +50,7 @@ _alltagtotable = {
     'Conductor' : 'conductor',
     'Date' : 'date',
     'Genre' : 'genre',
-    'Group' : 'contentgroup',
+    'Group' : 'cgroup', # can't have a table named group
     'Label' : 'label',
     'Lyricist' : 'lyricist',
     'Orchestra' : 'orchestra',
@@ -66,9 +67,32 @@ g_indextags = []
 g_tagdisplaytag = {}
 g_tagtotable = {}
 
+def getIndexTags():
+    return g_indextags
+def getTagDisplayTag():
+    return g_tagdisplaytag
+def getTagToTable():
+    return g_tagtotable
+
 def _clid(table):
     return table + '_id'
 
+# Partly duplicated from rclaudio, see comments about date formats
+# there.  The date field is a bit of a mess because up to some point
+# recoll had an alias from date to dmtime, which resulted in no date
+# field and a calendar date (instead of uxtime) in dmtime. So we take
+# the date value from 'date' if set, else 'dmtime', and then process
+# it here. All reasonable date formats begin with YYYY[-MM[-DD]] and
+# this is the part we keep.
+def parsedate(dt):
+    if len(dt) > 10:
+        dt = dt[0:10]
+    l = dt.split('-')
+    if len(l) > 3 or len(l) == 2 or len(l[0]) != 4 or l[0] == '0000':
+        return None
+    #uplog("parsedate: -> %s" % dt)
+    return dt
+
 # Create an empty db.
 #
 # There is then one table for each tag (Artist, Genre, Date,
@@ -142,6 +166,10 @@ def _prepareTags():
     global g_tagtotable
     global g_indextags
 
+    g_indextags = []
+    g_tagdisplaytag = {}
+    g_tagtotable = {}
+    
     indextagsp = uprclinit.g_minimconfig.getindextags()
     itemtags = uprclinit.g_minimconfig.getitemtags()
     if not indextagsp:
@@ -198,10 +226,10 @@ def _auxtableinsert(conn, tb, value):
 # should not arrive here, but let's play it safe.
 def _tracknofordoc(doc):
     try:
-        return int(doc.tracknumber.split('/')[0])
+        return int(doc["tracknumber"].split('/')[0])
     except:
-        #uplog("_tracknofordoc: doc.tracknumber %s title %s url %s" %
-        #      (doc.tracknumber, doc.title, doc.url))
+        #uplog("_tracknofordoc: doc["tracknumber"] %s title %s url %s" %
+        #      (doc["tracknumber"], doc["title"], doc["url"]))
         return 1
 
 
@@ -217,22 +245,22 @@ def _maybecreatealbum(conn, doc, trackartid):
     c = conn.cursor()
     folder = docfolder(doc).decode('utf-8', errors = 'replace')
 
-    album = getattr(doc, 'album', None)
+    album = doc["album"]
     if not album:
         album = uprclutils.basename(folder)
-        #uplog("Using %s for alb MIME %s title %s" % (album,doc.mtype,doc.url))
+        #uplog("Using %s for alb MIME %s title %s" % (album,doc["mtype"],doc["url"]))
 
-    if doc.albumartist:
-        albartist_id = _auxtableinsert(conn, 'artist', doc.albumartist)
+    if doc["albumartist"]:
+        albartist_id = _auxtableinsert(conn, 'artist', doc["albumartist"])
     else:
         albartist_id = None
 
     # See if there is a discnumber, either explicit or from album
     # title
     discnumber = None
-    if doc.discnumber:
+    if doc["discnumber"]:
         try:
-            discnumber = int(doc.discnumber)
+            discnumber = int(doc["discnumber"])
             #uplog("discnumber %d folder %s" % (discnumber, folder))
         except:
             pass
@@ -285,11 +313,13 @@ def _maybecreatealbum(conn, doc, trackartid):
                 c.execute('''UPDATE albums SET albtartist = ?
                     WHERE album_id = ?''', (trackartid, album_id))
     else:
+        arturi = uprclutils.docarturi(doc, uprclinit.getHttphp(), uprclinit.getPathPrefix())
+
         c.execute('''INSERT INTO
             albums(albtitle, albfolder, artist_id, albdate, albarturi,
             albtdisc, albartok, albtartist) VALUES (?,?,?,?,?,?,?,?)''',
-                  (album, folder, albartist_id, doc.date,
-                   doc.albumarturi, discnumber, 1, trackartid))
+                  (album, folder, albartist_id, doc["date"],
+                   arturi, discnumber, 1, trackartid))
         album_id = c.lastrowid
         #uplog("Created album %d %s disc %s artist %s folder %s" %
         #      (album_id, album, discnumber, albartist_id, folder))
@@ -324,7 +354,27 @@ def _setalbumartists(conn):
             c1.execute('''UPDATE albums SET artist_id = ?
                 WHERE album_id = ?''', (r[1], r[0]))
 
-
+# Update the recoll index for the albums
+def _albumstorecoll(conn):
+    rcldb = recoll.connect(confdir=uprclinit.getRclConfdir(), writable=True)
+    c = conn.cursor()
+    #                 0        1          2          3         4         5
+    stmt = '''SELECT album_id, albfolder, albtitle, albarturi, albdate, artist.value
+      FROM albums LEFT JOIN artist ON artist.artist_id = albums.artist_id
+      WHERE albtdisc is NULL'''
+    c.execute(stmt, ())
+    for r in c:
+        udi = "albid" + str(r[0])
+        doc = recoll.Doc()
+        doc["album"] = r[2]
+        doc["title"] = r[2]
+        doc["mtype"] = "inode/directory"
+        if r[5]:
+            doc["albumartist"] = r[5]
+        doc["url"] = "file://" + r[1]
+        uplog("_albumstorecoll: creating album for: %s" % doc["album"])
+        rcldb.addOrUpdate(udi, doc)
+    
 # Check that the numbers are sequential
 def _checkseq(seq):
     num = seq[0]
@@ -443,6 +493,7 @@ def recolltosql(conn, rcldocs):
 
     _createsqdb(conn)
     tabtorclfield = _prepareTags()
+    #uplog("Tagscreate: tabtorclfield: %s"%tabtorclfield)
 
     maxcnt = 0
     totcnt = 0
@@ -455,37 +506,44 @@ def recolltosql(conn, rcldocs):
             time.sleep(0)
         
         # No need to include non-audio or non-tagged types
-        if doc.mtype not in audiomtypes or doc.mtype == 'inode/directory' \
-               or doc.mtype == 'audio/x-mpegurl':
+        if doc["mtype"] not in audiomtypes or doc["mtype"] == 'inode/directory' \
+               or doc["mtype"] == 'audio/x-mpegurl':
             continue
 
         # Do the artist apart from the other attrs, as we need the
         # value for album creation.
-        if doc.albumartist:
-            trackartid = _auxtableinsert(conn, 'artist', doc.albumartist)
-        elif doc.artist:
-            trackartid = _auxtableinsert(conn, 'artist', doc.artist)
+        if doc["albumartist"]:
+            trackartid = _auxtableinsert(conn, 'artist', doc["albumartist"])
+        elif doc["artist"]:
+            trackartid = _auxtableinsert(conn, 'artist', doc["artist"])
         else:
             trackartid = None
         album_id, albartist_id = _maybecreatealbum(conn, doc, trackartid)
         
         trackno = _tracknofordoc(doc)
 
-        if doc.url.find('file://') == 0:
-            path = doc.url[7:]
+        if doc["url"].find('file://') == 0:
+            path = doc["url"][7:]
         else:
             path = ''
 
         # Set base values for column names, values list, placeholders
         columns = ['docidx','album_id','trackno','title','path','artist_id']
-        values = [docidx, album_id, trackno, doc.title, path, trackartid]
+        values = [docidx, album_id, trackno, doc["title"], path, trackartid]
         placehold = ['?', '?', '?', '?', '?', '?']
         # Append data for each auxiliary table if the doc has a value
         # for the corresponding field (else let SQL set a dflt/null value)
         for tb, rclfld in tabtorclfield:
             if tb == 'artist': # already done
                 continue
-            value = getattr(doc, rclfld, None)
+            value = doc[rclfld]
+            # See comment in parsedate
+            if rclfld == 'date':
+                if not value:
+                    value = doc['dmtime']
+                if value:
+                    value = parsedate(value)
+
             if not value:
                 continue
             rowid = _auxtableinsert(conn, tb, value)
@@ -497,7 +555,7 @@ def recolltosql(conn, rcldocs):
         stmt='INSERT INTO tracks(' + ','.join(columns) + \
               ') VALUES(' + ','.join(placehold) + ')'
         c.execute(stmt, values)
-        #uplog(doc.title)
+        #uplog(doc["title"])
 
     ## End Big doc loop
 
@@ -507,4 +565,5 @@ def recolltosql(conn, rcldocs):
     end = timer()
     uplog("recolltosql: processed %d docs in %.2f Seconds" %
           (totcnt, end-start))
-
+    _albumstorecoll(conn)
+    
