@@ -45,11 +45,6 @@ MPDCli::MPDCli(const string& host, int port, const string& pass)
     std::unique_lock<std::mutex> lock(m_mutex);
 
     regcomp(&m_tpuexpr, "^[[:alpha:]]+://.+", REG_EXTENDED|REG_NOSUB);
-    if (!openconn()) {
-        return;
-    }
-    m_have_addtagid = checkForCommand("addtagid");
-
     g_config->get("onstart", m_onstart);
     g_config->get("onplay", m_onplay);
     g_config->get("onpause", m_onpause);
@@ -61,6 +56,11 @@ MPDCli::MPDCli(const string& host, int port, const string& pass)
     stringToStrings(scratch, m_getexternalvolume);
     m_timeoutms = g_config->getInt("mpdtimeoutms", 2000);
     m_externalvolumecontrol = g_config->getBool("externalvolumecontrol", false);
+
+    if (!openconn()) {
+        return;
+    }
+    m_have_addtagid = checkForCommand("addtagid");
     updStatus();
 }
 
@@ -109,7 +109,9 @@ bool MPDCli::openconn()
     }
 
     if (mpd_connection_get_error(m_conn) != MPD_ERROR_SUCCESS) {
-        showError("MPDCli::openconn");
+        // Don't call showError() from here it calls us back !
+        LOGERR("MPDCli::openconn: mpd_connection_new failed: "
+               <<  mpd_connection_get_error_message(m_conn) << "\n");
         closeconn();
         return false;
     }
@@ -129,13 +131,18 @@ bool MPDCli::openconn()
     LOGDEB("MPDCLi::openconn: mpd protocol version: " << m_stat.versmajor
            << "." << m_stat.versminor << "." << m_stat.verspatch << endl);
 
+    startEventLoop();
     return true;
 }
 
 bool MPDCli::startEventLoop()
 {
     LOGDEB("MPDCli::startEventLoop\n");
+    std::lock_guard<std::mutex> lck(m_idlemutex);
     if (nullptr == m_idleconn) {
+        if (m_idlethread.joinable()) {
+            m_idlethread.join();
+        }
         m_idlethread = std::thread(bind(&MPDCli::eventLoop, this));
     } else {
         LOGINF("MPDCli::startEventLoop: already started\n");
@@ -146,10 +153,14 @@ bool MPDCli::startEventLoop()
 void MPDCli::stopEventLoop()
 {
     LOGDEB("MPDCli::stopEventLoop\n");
+    std::unique_lock<std::mutex> lck(m_idlemutex);
     if (m_idleconn) {
+        m_idleneedstop = true;
         mpd_send_noidle(m_idleconn);
+        lck.unlock();
         m_idlethread.join();
         m_idlethread = std::thread{};
+        m_idleneedstop = false;
     }
 }
 
@@ -193,55 +204,92 @@ static int o_idle_mask =
 
 bool MPDCli::eventLoop()
 {
-    m_idleconn = mpd_connection_new(m_host.c_str(), m_port, m_timeoutms);
-    if (nullptr == m_idleconn) {
-        LOGERR("MPDCli::eventloop: could not open connection\n");
-        return false;
+top:
+    for (;;) {
+        std::lock_guard<std::mutex> lck(m_idlemutex);
+        m_idleconn = mpd_connection_new(m_host.c_str(), m_port, m_timeoutms);
+        if (nullptr == m_idleconn) {
+            LOGINF("MPDCli::eventloop: could not open connection\n");
+            std::this_thread::sleep_for(std::chrono::milliseconds(2000));
+            continue;
+        }
+        if (!m_password.empty()) {
+            if (!mpd_run_password(m_idleconn, m_password.c_str())) {
+                LOGERR("MPDCli::eventloop: password wrong ?\n");
+                mpd_connection_free(m_idleconn);
+                m_idleconn = nullptr;
+                return false;
+            }
+        }
+        break;
     }
+    MpdStatus::State st;
     {
         std::unique_lock<std::mutex> lock(m_mutex);
         updStatus();
+        st = m_stat.state;
     }
-    pollerCtl(m_stat.state);
+    pollerCtl(st);
     for (;;) {
         enum mpd_idle mask = 
             mpd_run_idle_mask(m_idleconn, (enum mpd_idle)o_idle_mask);
         if (mask == 0) {
             LOGERR("MPDCli::eventloop: mpd_run_idle_mask returned 0\n");
-            break;
+            // This can happen if mpd went awol, or if we're asked to
+            // stop. In the first case, try to reconnect, else exit
+            // the loop
+            if (m_idleneedstop) {
+                m_idleneedstop = false;
+                break;
+            }
+            std::this_thread::sleep_for(std::chrono::milliseconds(2000));
+            goto top;
         }
         LOGDEB0("MPDCli::eventloop: mpd_run_idle_mask returned " << std::hex <<
                 mask << std::dec << "\n");
         {
             std::unique_lock<std::mutex> lock(m_mutex);
             updStatus();
+            st = m_stat.state;
         }
-        pollerCtl(m_stat.state);
-        for (auto& sub : m_subs) {
-            if (sub.first & mask) {
-                sub.second(&m_stat);
+        pollerCtl(st);
+
+        {
+            std::lock_guard<std::mutex> lock(m_callbackmutex);
+            for (auto& sub : m_subs) {
+                if (sub.first & mask) {
+                    sub.second(&m_stat);
+                }
             }
         }
+        // Rate-limiting the events we trigger improves big list
+        // insertion performance a lot because it decreases the number
+        // of times we have to update the list
+        std::this_thread::sleep_for(std::chrono::milliseconds(1000));
     }
+    
     pollerCtl(MpdStatus::MPDS_STOP);
-    if (m_idleconn) {
-        mpd_connection_free(m_idleconn);
-        m_idleconn = nullptr;
-    }
+
+    std::lock_guard<std::mutex> lck(m_idlemutex);
+    mpd_connection_free(m_idleconn);
+    m_idleconn = nullptr;
     return false;
 }
 
 void MPDCli::timepoller()
 {
     for (;;) {
-        LOGDEB0("MPDCli::timepoller\n");
+        LOGDEB1("MPDCli::timepoller\n");
         {
             std::unique_lock<std::mutex> lock(m_mutex);
             updStatus();
         }
-        for (auto& sub : m_subs) {
-            if (sub.first & MpdPlayerEvt) {
-                sub.second(&m_stat);
+        {
+            std::lock_guard<std::mutex> lock(m_callbackmutex);
+            for (auto& sub : m_subs) {
+                if (sub.first & MpdPlayerEvt) {
+                    sub.second(&m_stat);
+                }
             }
         }
 
@@ -262,6 +310,7 @@ void MPDCli::shouldExit()
 
 bool MPDCli::subscribe(int mask, evtFunc func)
 {
+    std::lock_guard<std::mutex> lock(m_callbackmutex);
     m_subs.push_back({mask, func});
     return true;
 }
@@ -285,10 +334,12 @@ bool MPDCli::showError(const string& who)
         LOGERR(who << " server error: " << 
                mpd_connection_get_server_error(m_conn) << endl);
     }
-
     if (error == MPD_ERROR_CLOSED)
         if (openconn())
             return true;
+
+    if (!mpd_connection_clear_error(m_conn))
+        openconn();
     return false;
 }
 
@@ -344,8 +395,8 @@ bool MPDCli::updStatus()
             if (mpds == 0) {
                 LOGERR("MPDCli::updStatus: can't get status" << endl);
                 showError("MPDCli::updStatus");
+                return false;
             }
-            return false;
         }
     }
     
@@ -488,11 +539,12 @@ bool MPDCli::checkForCommand(const string& cmdname)
     return found;
 }
 
-const MpdStatus& MPDCli::getStatus()
+MpdStatus MPDCli::getStatus()
 {
     std::unique_lock<std::mutex> lock(m_mutex);
     // We used to updstatus() here, but the status should now be
     // always up to date because of the idle loop.
+    // We have to return a copy, as we are going to release the lock.
     return m_stat;
 }
 
@@ -1044,7 +1096,8 @@ bool MPDCli::getQueueSongs(vector<mpd_song*>& songs)
     }
     
     if (!mpd_response_finish(m_conn)) {
-        LOGERR("MPDCli::getQueueSongs: mpd_list_queue_meta failed"<< endl);
+        showError("MPDCli::getQueueSongs: mpd_list_queue_meta:finish");
+        freeSongs(songs);
         return false;
     }
     LOGDEB("MPDCli::getQueueSongs: " << songs.size() << " songs " << endl);
