@@ -42,6 +42,7 @@
 #include "pathut.h"
 #include "conftree.h"
 #include "ohcredentials.hxx"
+#include "urlmorph.hxx"
 
 using namespace std;
 using namespace std::placeholders;
@@ -54,6 +55,12 @@ static const string sIdProduct("urn:av-openhome-org:serviceId:Playlist");
 // mpc/mpd control. If set we don't touch it.
 static bool keepconsume;
 
+// For OHCreds/morphSpecialUrl: the media server, which is used to run
+// the microhttpd and for getting the real media URLs, must run on
+// this host (for one thing the creds are passed either through shared
+// memory or through a local file).
+static string upnphost;
+
 // Playlist is the default oh service, so it's active when starting up
 OHPlaylist::OHPlaylist(UpMpd *dev,  UpMpdOpenHome *udev, unsigned int cssleep)
     : OHService(sTpProduct, sIdProduct, "OHPlaylist.xml", dev, udev),
@@ -120,6 +127,9 @@ OHPlaylist::OHPlaylist(UpMpd *dev,  UpMpdOpenHome *udev, unsigned int cssleep)
     m_dev->getmpdcli()->subscribe(
         MPDCli::MpdQueueEvt|MPDCli::MpdPlayerEvt|MPDCli::MpdOptsEvt,
         std::bind(&OHService::onEvent, this, _1));
+
+    unsigned short usport;
+    udev->ipv4(&upnphost, &usport);
 }
 
 static const int tracksmax = 16384;
@@ -129,7 +139,14 @@ static const int tracksmax = 16384;
 static string translateIdArray(const vector<UpSong>& in)
 {
     string out1;
+    out1.reserve(in.size() * 4);
+
+#undef TIA_SHOW_PLAIN_ARRAY
+#ifdef TIA_SHOW_PLAIN_ARRAY
     string sdeb;
+    sdeb.reserve(in.size() * 7);
+#endif // TIA_SHOW_PLAIN_ARRAY
+    
     for (auto us = in.begin(); us != in.end(); us++) {
         unsigned int val = us->mpdid;
         if (val) {
@@ -138,15 +155,19 @@ static string translateIdArray(const vector<UpSong>& in)
             out1 += (unsigned char) ((val & 0x0000ff00) >> 8);
             out1 += (unsigned char) ((val & 0x000000ff));
         }
+#ifdef TIA_SHOW_PLAIN_ARRAY
         sdeb += SoapHelp::i2s(val) + " ";
+#endif
     }
-    LOGDEB("OHPlaylist::translateIdArray: current ids: " << sdeb << endl);
+#ifdef TIA_SHOW_PLAIN_ARRAY
+    LOGINF("OHPlaylist::translateIdArray: current ids: " << sdeb << endl);
+#endif
     return base64_encode(out1);
 }
 
 bool OHPlaylist::makeIdArray(string& out)
 {
-    //LOGDEB1("OHPlaylist::makeIdArray\n");
+    LOGDEB1("OHPlaylist::makeIdArray\n");
     const MpdStatus &mpds = m_dev->getMpdStatus();
 
     if (mpds.qvers == m_mpdqvers) {
@@ -298,33 +319,15 @@ bool OHPlaylist::makestate(unordered_map<string, string> &st)
     return true;
 }
 
-void OHPlaylist::refreshState()
-{
-    m_mpdqvers = -1;
-    unordered_map<string, string> st;
-    makestate(st);
-}
-
-void OHPlaylist::maybeWakeUp(bool ok)
-{
-    if (ok && m_dev) {
-        onEvent(nullptr);
-    }
-}
-
 void OHPlaylist::setActive(bool onoff)
 {
     if (onoff) {
         m_dev->getmpdcli()->clearQueue();
         m_dev->getmpdcli()->restoreState(m_mpdsavedstate);
-        if (!keepconsume)
-            m_dev->getmpdcli()->consume(false);
-        m_dev->getmpdcli()->single(false);
-        refreshState();
-        maybeWakeUp(true);
+        onEvent(nullptr);
         m_active = true;
     } else {
-        m_mpdqvers = -1;
+        std::lock_guard<std::mutex> lock(m_statemutex);
         makestate(m_upnpstate);
         m_dev->getmpdcli()->saveState(m_mpdsavedstate);
         iStop();
@@ -336,14 +339,13 @@ int OHPlaylist::play(const SoapIncoming& sc, SoapOutgoing& data)
 {
     LOGDEB("OHPlaylist::play" << endl);
     if (!m_active && m_udev->getohpr()) {
-        m_udev->getohpr()->iSetSourceIndexByName("Playlist");
+        m_udev->getohpr()->iSetSourceIndexByName(OHPlaylistSourceName);
     }
     bool ok;
     if (!keepconsume) 
         m_dev->getmpdcli()->consume(false);
     m_dev->getmpdcli()->single(false);
     ok = m_dev->getmpdcli()->play();
-    maybeWakeUp(ok);
     return ok ? UPNP_E_SUCCESS : UpnpService::UPNP_ACTION_FAILED;
 }
 
@@ -351,14 +353,12 @@ int OHPlaylist::pause(const SoapIncoming& sc, SoapOutgoing& data)
 {
     LOGDEB("OHPlaylist::pause" << endl);
     bool ok = m_dev->getmpdcli()->pause(true);
-    maybeWakeUp(ok);
     return ok ? UPNP_E_SUCCESS : UPNP_E_INTERNAL_ERROR;
 }
 
 int OHPlaylist::iStop()
 {
     bool ok = m_dev->getmpdcli()->stop();
-    maybeWakeUp(ok);
     return ok ? UPNP_E_SUCCESS : UPNP_E_INTERNAL_ERROR;
 }
 int OHPlaylist::stop(const SoapIncoming& sc, SoapOutgoing& data)
@@ -375,7 +375,6 @@ int OHPlaylist::next(const SoapIncoming& sc, SoapOutgoing& data)
     }
     LOGDEB("OHPlaylist::next" << endl);
     bool ok = m_dev->getmpdcli()->next();
-    maybeWakeUp(ok);
     return ok ? UPNP_E_SUCCESS : UPNP_E_INTERNAL_ERROR;
 }
 
@@ -387,7 +386,6 @@ int OHPlaylist::previous(const SoapIncoming& sc, SoapOutgoing& data)
     }
     LOGDEB("OHPlaylist::previous" << endl);
     bool ok = m_dev->getmpdcli()->previous();
-    maybeWakeUp(ok);
     return ok ? UPNP_E_SUCCESS : UPNP_E_INTERNAL_ERROR;
 }
 
@@ -402,7 +400,6 @@ int OHPlaylist::setRepeat(const SoapIncoming& sc, SoapOutgoing& data)
     bool ok = sc.get("Value", &onoff);
     if (ok) {
         ok = m_dev->getmpdcli()->repeat(onoff);
-        maybeWakeUp(ok);
     }
     return ok ? UPNP_E_SUCCESS : UPNP_E_INTERNAL_ERROR;
 }
@@ -432,7 +429,6 @@ int OHPlaylist::setShuffle(const SoapIncoming& sc, SoapOutgoing& data)
         // Note that mpd shuffle shuffles the playlist, which is different
         // from playing at random
         ok = m_dev->getmpdcli()->random(onoff);
-        maybeWakeUp(ok);
     }
     return ok ? UPNP_E_SUCCESS : UPNP_E_INTERNAL_ERROR;
 }
@@ -460,7 +456,6 @@ int OHPlaylist::seekSecondAbsolute(const SoapIncoming& sc, SoapOutgoing& data)
     bool ok = sc.get("Value", &seconds);
     if (ok) {
         ok = m_dev->getmpdcli()->seek(seconds);
-        maybeWakeUp(ok);
     }
     return ok ? UPNP_E_SUCCESS : UPNP_E_INTERNAL_ERROR;
 }
@@ -484,7 +479,6 @@ int OHPlaylist::seekSecondRelative(const SoapIncoming& sc, SoapOutgoing& data)
         } else {
             ok = false;
         }
-        maybeWakeUp(ok);
     }
     return ok ? UPNP_E_SUCCESS : UPNP_E_INTERNAL_ERROR;
 }
@@ -521,7 +515,7 @@ int OHPlaylist::seekId(const SoapIncoming& sc, SoapOutgoing& data)
         // If I'm not active, the ids in the playlist are those of
         // another service (e.g. radio). After activating myself and
         // restoring the playlist, the input id needs to be mapped.
-        m_udev->getohpr()->iSetSourceIndexByName("Playlist");
+        m_udev->getohpr()->iSetSourceIndexByName(OHPlaylistSourceName);
         id = idFromOldId(id);
         if (id < 0) {
             return UPNP_E_INTERNAL_ERROR;
@@ -531,7 +525,6 @@ int OHPlaylist::seekId(const SoapIncoming& sc, SoapOutgoing& data)
         m_dev->getmpdcli()->consume(false);
     m_dev->getmpdcli()->single(false);
     bool ok = m_dev->getmpdcli()->playId(id);
-    maybeWakeUp(ok);
     return ok ? UPNP_E_SUCCESS : UPNP_E_INTERNAL_ERROR;
 }
 
@@ -543,7 +536,7 @@ int OHPlaylist::seekIndex(const SoapIncoming& sc, SoapOutgoing& data)
     // Unlike seekid, this should work as the indices are restored by
     // mpdcli restorestate
     if (!m_active && m_udev->getohpr()) {
-        m_udev->getohpr()->iSetSourceIndexByName("Playlist");
+        m_udev->getohpr()->iSetSourceIndexByName(OHPlaylistSourceName);
     }
     int pos;
     bool ok = sc.get("Value", &pos);
@@ -552,7 +545,6 @@ int OHPlaylist::seekIndex(const SoapIncoming& sc, SoapOutgoing& data)
             m_dev->getmpdcli()->consume(false);
         m_dev->getmpdcli()->single(false);
         ok = m_dev->getmpdcli()->play(pos);
-        maybeWakeUp(ok);
     }
     return ok ? UPNP_E_SUCCESS : UPNP_E_INTERNAL_ERROR;
 }
@@ -573,14 +565,26 @@ int OHPlaylist::id(const SoapIncoming& sc, SoapOutgoing& data)
 
 bool OHPlaylist::cacheFind(const string& uri, string& meta)
 {
+    std::lock_guard<std::mutex> lock(m_statemutex);
     auto cached = m_metacache.find(uri);
     if (cached != m_metacache.end()) {
         meta = cached->second;
+        LOGDEB1("OHPlaylist::cacheFind: " << uri << " -> " << meta << "\n");
         return true;
     }
+    LOGDEB1("OHPlaylist::cacheFind: " << uri << " not found\n");
     return false;
 }
 
+bool OHPlaylist::cacheSet(const string& uri, const string& meta)
+{
+    std::lock_guard<std::mutex> lock(m_statemutex);
+    LOGDEB1("OHPlaylist::cacheSet: " << uri << " -> " << meta << "\n");
+    m_metacache[uri] = meta;
+    m_cachedirty = true;
+    return true;
+}
+
 // Report the uri and metadata for a given track id. 
 // Returns a 800 fault code if the given id is not in the playlist. 
 int OHPlaylist::ohread(const SoapIncoming& sc, SoapOutgoing& data)
@@ -599,13 +603,9 @@ int OHPlaylist::ohread(const SoapIncoming& sc, SoapOutgoing& data)
             LOGERR("OHPlaylist::ohread: statsong failed for " << id << endl);
             return UPNP_E_INTERNAL_ERROR;
         }
-        auto cached = m_metacache.find(song.rsrc.uri);
-        if (cached != m_metacache.end()) {
-            metadata = cached->second;
-        } else {
+        if (!cacheFind(song.rsrc.uri, metadata)) {
             metadata = didlmake(song);
-            m_metacache[song.rsrc.uri] = metadata;
-            m_cachedirty = true;
+            cacheSet(song.rsrc.uri, metadata);
         }
     } else {
         LOGDEB("OHPlaylist::read: not active: using saved queue\n");
@@ -620,7 +620,7 @@ int OHPlaylist::ohread(const SoapIncoming& sc, SoapOutgoing& data)
             return UPNP_E_INTERNAL_ERROR;
         }
     }
-    data.addarg("Uri", SoapHelp::xmlQuote(song.rsrc.uri));
+    data.addarg("Uri", song.rsrc.uri);
     data.addarg("Metadata", metadata);
     return UPNP_E_SUCCESS;
 }
@@ -657,33 +657,20 @@ int OHPlaylist::readList(const SoapIncoming& sc, SoapOutgoing& data)
             UpSong song;
             if (m_active) {
                 if (!m_dev->getmpdcli()->statSong(song, id, true)) {
-                    LOGDEB("OHPlaylist::readList:stat failed for " <<
-                           id <<endl);
+                    LOGDEB("OHPlaylist::readList:stat failed for " << id <<"\n");
                     continue;
                 }
-                auto mit = m_metacache.find(song.rsrc.uri);
-                if (mit != m_metacache.end()) {
-                    LOGDEB1("OHPlaylist::readList: meta for id " << id << " uri "
-                            << song.rsrc.uri << " found in cache " << endl);
-                    metadata = SoapHelp::xmlQuote(mit->second);
-                } else {
-                    LOGDEB("OHPlaylist::readList: meta for id " << id << " uri "
-                           << song.rsrc.uri << " not found " << endl);
+                if (!cacheFind(song.rsrc.uri, metadata)) {
                     metadata = didlmake(song);
-                    m_metacache[song.rsrc.uri] = metadata;
-                    m_cachedirty = true;
-                    metadata = SoapHelp::xmlQuote(metadata);
+                    cacheSet(song.rsrc.uri, metadata);
                 }
             } else {
                 LOGDEB("OHPlaylist::readList: not active: using saved queue\n");
                 for (const auto& entry : m_mpdsavedstate.queue) {
                     if (entry.mpdid == id) {
                         song = entry;
-                        auto mit = m_metacache.find(song.rsrc.uri);
-                        if (mit != m_metacache.end()) {
-                            metadata = SoapHelp::xmlQuote(mit->second);
-                        } else {
-                            metadata = SoapHelp::xmlQuote(didlmake(song));
+                        if (!cacheFind(song.rsrc.uri, metadata)) {
+                            metadata = didlmake(song);
                         }
                     }
                 }
@@ -697,7 +684,7 @@ int OHPlaylist::readList(const SoapIncoming& sc, SoapOutgoing& data)
             out += "</Id><Uri>";
             out += SoapHelp::xmlQuote(song.rsrc.uri);
             out += "</Uri><Metadata>";
-            out += metadata;
+            out += SoapHelp::xmlQuote(metadata);
             out += "</Metadata></Entry>";
         }
         out += "</TrackList>";
@@ -707,20 +694,6 @@ int OHPlaylist::readList(const SoapIncoming& sc, SoapOutgoing& data)
     return ok ? UPNP_E_SUCCESS : UPNP_E_INTERNAL_ERROR;
 }
 
-bool OHPlaylist::ireadList(const vector<int>& ids, vector<UpSong>& songs)
-{
-    for (auto it = ids.begin(); it != ids.end(); it++) {
-        UpSong song;
-        if (!m_dev->getmpdcli()->statSong(song, *it, true)) {
-            LOGDEB("OHPlaylist::readList:stat failed for " << *it << endl);
-            continue;
-        }
-        songs.push_back(song);
-    }
-    return true;
-}
-
-
 // Adds the given uri and metadata as a new track to the playlist. 
 // Set the AfterId argument to 0 to insert a track at the start of the
 // playlist.
@@ -744,16 +717,16 @@ int OHPlaylist::insert(const SoapIncoming& sc, SoapOutgoing& data)
     }
 
     // Maybe transform a qobuz:// or tidal:// uri if we're doing this
-    // isStreaming is used to disable content format check in this
+    // forcenocheck is used to disable content format check in this
     // case (there is no valid protocolinfo in general).
-    bool isStreaming;
-    if (!OHCredsMaybeMorphSpecialUri(uri, isStreaming)) {
+    bool forcenocheck;
+    if (!morphSpecialUrl(uri, forcenocheck, upnphost)) {
         LOGERR("OHPlaylist::insert: bad uri: " << uri << endl);
         return UPNP_E_INVALID_PARAM;
     }
         
     if (!m_active) {
-        m_udev->getohpr()->iSetSourceIndexByName("Playlist");
+        m_udev->getohpr()->iSetSourceIndexByName(OHPlaylistSourceName);
         afterid = idFromOldId(afterid);
         if (afterid < 0) {
             return UPNP_E_INTERNAL_ERROR;
@@ -764,12 +737,11 @@ int OHPlaylist::insert(const SoapIncoming& sc, SoapOutgoing& data)
            uri << " Metadata " << metadata << endl);
 
     int newid;
-    ok = insertUri(afterid, uri, metadata, &newid, isStreaming);
+    ok = insertUri(afterid, uri, metadata, &newid, forcenocheck);
     if (ok) {
         data.addarg("NewId", SoapHelp::i2s(newid));
         LOGDEB("OHPlaylist::insert: new id: " << newid << endl);
     }
-    maybeWakeUp(ok);
     return ok ? UPNP_E_SUCCESS : UPNP_E_INTERNAL_ERROR;
 }
 
@@ -779,7 +751,7 @@ bool OHPlaylist::insertUri(int afterid, const string& uri,
     LOGDEB1("OHPlaylist::insertUri: " << uri << endl);
     if (!m_active) {
         LOGERR("OHPlaylist::insertUri: not active" << endl);
-        m_udev->getohpr()->iSetSourceIndexByName("Playlist");
+        m_udev->getohpr()->iSetSourceIndexByName(OHPlaylistSourceName);
         return false;
     }
 
@@ -789,11 +761,11 @@ bool OHPlaylist::insertUri(int afterid, const string& uri,
                " metadata " << metadata);
         return false;
     }
+
+    cacheSet(uri, metadata);
+
     int id = m_dev->getmpdcli()->insertAfterId(uri, afterid, metaformpd);
     if (id != -1) {
-        m_metacache[uri] = metadata;
-        m_cachedirty = true;
-        m_mpdqvers = -1;
         if (newid)
             *newid = id;
         return true;
@@ -810,7 +782,7 @@ int OHPlaylist::deleteId(const SoapIncoming& sc, SoapOutgoing& data)
         return UPNP_E_INVALID_PARAM;
     }
     if (!m_active) {
-        m_udev->getohpr()->iSetSourceIndexByName("Playlist");
+        m_udev->getohpr()->iSetSourceIndexByName(OHPlaylistSourceName);
         id = idFromOldId(id);
         if (id < 0) {
             // Error was logged by idFromOldId
@@ -825,8 +797,6 @@ int OHPlaylist::deleteId(const SoapIncoming& sc, SoapOutgoing& data)
         m_dev->getmpdcli()->stop();
     }
     bool ok = m_dev->getmpdcli()->deleteId(id);
-    m_mpdqvers = -1;
-    maybeWakeUp(ok);
     return ok ? UPNP_E_SUCCESS : UPNP_E_INTERNAL_ERROR;
 }
 
@@ -834,11 +804,9 @@ int OHPlaylist::deleteAll(const SoapIncoming& sc, SoapOutgoing& data)
 {
     LOGDEB("OHPlaylist::deleteAll" << endl);
     if (!m_active && m_udev->getohpr()) {
-        m_udev->getohpr()->iSetSourceIndexByName("Playlist");
+        m_udev->getohpr()->iSetSourceIndexByName(OHPlaylistSourceName);
     }
     bool ok = m_dev->getmpdcli()->clearQueue();
-    m_mpdqvers = -1;
-    maybeWakeUp(ok);
     return ok ? UPNP_E_SUCCESS : UPNP_E_INTERNAL_ERROR;
 }
 
@@ -849,12 +817,15 @@ int OHPlaylist::tracksMax(const SoapIncoming& sc, SoapOutgoing& data)
     return UPNP_E_SUCCESS;
 }
 
-
+// Call with state lock held
 bool OHPlaylist::iidArray(string& idarray, int *token)
 {
     LOGDEB("OHPlaylist::idArray (internal)" << endl);
     unordered_map<string, string> st;
-    makestate(st);
+    {
+        std::lock_guard<std::mutex> lock(m_statemutex);
+        makestate(st);
+    }
     idarray = st["IdArray"];
     if (token) {
         if (m_active) {
@@ -884,9 +855,22 @@ int OHPlaylist::idArray(const SoapIncoming& sc, SoapOutgoing& data)
     return UPNP_E_INTERNAL_ERROR;
 }
 
+bool OHPlaylist::ireadList(const vector<int>& ids, vector<UpSong>& songs)
+{
+    for (auto it = ids.begin(); it != ids.end(); it++) {
+        UpSong song;
+        if (!m_dev->getmpdcli()->statSong(song, *it, true)) {
+            LOGDEB("OHPlaylist::readList:stat failed for " << *it << endl);
+            continue;
+        }
+        songs.push_back(song);
+    }
+    return true;
+}
 
 bool OHPlaylist::urlMap(unordered_map<int, string>& umap)
 {
+    std::lock_guard<std::mutex> lock(m_statemutex);
     LOGDEB1("OHPlaylist::urlMap\n");
     string sarray; 
     if (iidArray(sarray, 0)) {
