@@ -46,8 +46,6 @@
 using namespace std;
 using namespace UPnPP;
 
-static const int minVolumeDelta = 5;
-
 static const string iconDesc(
     "<iconList>"
     "  <icon>"
@@ -70,9 +68,9 @@ UpMpdMediaRenderer::UpMpdMediaRenderer(
                   "urn:schemas-upnp-org:device:MediaRenderer:1")
 {
     bool noavt = (0 != (m_upmpd->getopts().options & UpMpd::upmpdNoAV));
-    m_avt = new UpMpdAVTransport(upmpd, this, noavt);
+    m_avt = new AVTransport(upmpd, this, noavt);
     m_services.push_back(m_avt);
-    m_services.push_back(new UpMpdRenderCtl(upmpd, this, noavt));
+    m_services.push_back(new RenderingControl(upmpd, this, noavt));
     m_services.push_back(new UpMpdConMan(this));
 }
 
@@ -161,8 +159,6 @@ UpMpdOpenHome::UpMpdOpenHome(
                   "urn:av-openhome-org:device:Source:1")
 {
     bool noavt = (0 != (m_upmpd->getopts().options & UpMpd::upmpdNoAV));
-    m_ohif = new OHInfo(m_upmpd, this, noavt);
-    m_services.push_back(m_ohif);
 
     m_services.push_back(new OHTime(m_upmpd, this));
     m_services.push_back(new OHVolume(m_upmpd, this));
@@ -175,8 +171,7 @@ UpMpdOpenHome::UpMpdOpenHome(
     m_ohpl = new OHPlaylist(m_upmpd, this, m_upmpd->getopts().ohmetasleep);
     m_services.push_back(m_ohpl);
     m_upmpd->setohpl(m_ohpl);
-    if (m_ohif)
-        m_ohif->setOHPL(m_ohpl);
+
     m_ohrd = new OHRadio(m_upmpd, this);
     if (m_ohrd && !m_ohrd->ok()) {
         delete m_ohrd;
@@ -184,6 +179,11 @@ UpMpdOpenHome::UpMpdOpenHome(
     }
     if (m_ohrd)
         m_services.push_back(m_ohrd);
+
+    m_ohif = new OHInfo(m_upmpd, this, noavt);
+    m_ohif->setOHPL(m_ohpl);
+    m_services.push_back(m_ohif);
+
     if (m_upmpd->getopts().options & UpMpd::upmpdOhReceiver) {
         struct OHReceiverParams parms;
         if (m_upmpd->getopts().schttpport)
@@ -217,17 +217,12 @@ UpMpdOpenHome::UpMpdOpenHome(
     m_services.push_back(m_ohpr);
 }
 
-// Note: if we ever need this to work without cxx11, there is this:
-// http://www.tutok.sk/fastgl/callback.html
 UpMpd::UpMpd(const string& hwaddr, const string& friendlyname,
              ohProductDesc_t& ohProductDesc, MPDCli *mpdcli, Options opts)
     : m_mpdcli(mpdcli),
       m_allopts(opts),
       m_mcachefn(opts.cachefn)
 {
-    // Note: the order is significant here as it will be used when
-    // calling the getStatus() methods, and we want AVTransport to
-    // update the mpd status for everybody
     if (0 == (opts.options & upmpdNoAV)) {
         std::string avfname{friendlyname + "-UPnP/AV"};
         g_config->get("avfriendlyname", avfname);
@@ -235,11 +230,17 @@ UpMpd::UpMpd(const string& hwaddr, const string& friendlyname,
         std::string deviceid =  std::string("uuid:") +
             LibUPnP::makeDevUUID(avfname + "xy3vhst39", hwaddr);
         m_av = new UpMpdMediaRenderer(this, deviceid, avfname);
+#if LIBUPNPP_AT_LEAST(0,21,0)
+        m_av->setProductVersion("Upmpdcli", g_upmpdcli_package_version.c_str());
+#endif
     }
     if (opts.options & upmpdDoOH) {
         std::string deviceid =  std::string("uuid:") +
             LibUPnP::makeDevUUID(friendlyname, hwaddr);
         m_oh = new UpMpdOpenHome(this, deviceid, friendlyname, ohProductDesc);
+#if LIBUPNPP_AT_LEAST(0,21,0)
+        m_oh->setProductVersion("Upmpdcli", g_upmpdcli_package_version.c_str());
+#endif
     }
 }
 
@@ -251,58 +252,31 @@ UpMpd::~UpMpd()
 //    }
 }
 
-const MpdStatus& UpMpd::getMpdStatus()
+MpdStatus UpMpd::getMpdStatus()
 {
     return m_mpdcli->getStatus();
 }
 
 int UpMpd::getvolume()
 {
-    return m_desiredvolume >= 0 ? m_desiredvolume : m_mpdcli->getVolume();
+    return m_mpdcli->getVolume();
 }
 
 bool UpMpd::setvolume(int volume)
 {
-    int previous_volume = m_mpdcli->getVolume();
-    int delta = previous_volume - volume;
-    if (delta < 0)
-        delta = -delta;
-    LOGDEB("UpMpd::setVolume: volume " << volume << " delta " << 
-           delta << endl);
-    bool ret{false};
-    if (delta >= minVolumeDelta) {
-        ret = m_mpdcli->setVolume(volume);
-        m_desiredvolume = -1;
-    } else {
-        m_desiredvolume = volume;
-    }
-    return ret;
+    return m_mpdcli->setVolume(volume);
 }
 
 bool UpMpd::flushvolume()
 {
-    bool ret{false};
-    if (m_desiredvolume >= 0) {
-        ret = m_mpdcli->setVolume(m_desiredvolume);
-        m_desiredvolume = -1;
-    }
-    return ret;
+    return true;
 }
 
 bool UpMpd::setmute(bool onoff)
 {
-    bool ret{false};
-    if (onoff) {
-        if (m_desiredvolume >= 0) {
-            m_mpdcli->setVolume(m_desiredvolume);
-            m_desiredvolume = -1;
-        }
-        ret = m_mpdcli->setVolume(0, true);
-    } else {
-        // Restore pre-mute
-        ret = m_mpdcli->setVolume(1, true);
-    }
-    return ret;
+    // See mpdcli.cxx for the special processing when the 2nd arg is true
+    return onoff ? m_mpdcli->setVolume(0, true) :
+        m_mpdcli->setVolume(1, true);
 }
 
 bool UpMpd::checkContentFormat(const string& uri, const string& didl,
